Stan Math Library: stan/math/prim/scal/prob/weibull_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

weibull_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_WEIBULL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_WEIBULL_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/fun/constants.hpp>
16 #include <stan/math/prim/scal/meta/include_summand.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <boost/random/weibull_distribution.hpp>
20 #include <boost/random/variate_generator.hpp>
21 #include <cmath>
22
23 namespace stan {
24
25 namespace math {
26
27 template <typename T_y, typename T_shape, typename T_scale>
28 typename return_type<T_y, T_shape, T_scale>::type
29 weibull_cdf_log(const T_y& y, const T_shape& alpha, const T_scale& sigma) {
30 typedef typename stan::partials_return_type<T_y, T_shape, T_scale>::type
31 T_partials_return;
32
33 static const char* function("stan::math::weibull_cdf_log");
34
35 using stan::math::check_positive_finite;
36 using stan::math::check_nonnegative;
37 using boost::math::tools::promote_args;
38 using stan::math::value_of;
39 using std::log;
40 using std::exp;
41
42 // check if any vectors are zero length
43 if (!(stan::length(y)
44 && stan::length(alpha)
45 && stan::length(sigma)))
46 return 0.0;
47
48 T_partials_return cdf_log(0.0);
49 check_nonnegative(function, "Random variable", y);
50 check_positive_finite(function, "Shape parameter", alpha);
51 check_positive_finite(function, "Scale parameter", sigma);
52
53 OperandsAndPartials<T_y, T_shape, T_scale>
54 operands_and_partials(y, alpha, sigma);
55
56 VectorView<const T_y> y_vec(y);
57 VectorView<const T_scale> sigma_vec(sigma);
58 VectorView<const T_shape> alpha_vec(alpha);
59 size_t N = max_size(y, sigma, alpha);
60 for (size_t n = 0; n < N; n++) {
61 const T_partials_return y_dbl = value_of(y_vec[n]);
62 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
63 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
64 const T_partials_return pow_ = pow(y_dbl / sigma_dbl, alpha_dbl);
65 const T_partials_return exp_ = exp(-pow_);
66 const T_partials_return cdf_ = 1.0 - exp_;
67
68 // cdf_log
69 cdf_log += log(cdf_);
70
71 // gradients
72 const T_partials_return rep_deriv = pow_ / (1.0 / exp_ - 1.0);
73 if (!is_constant_struct<T_y>::value)
74 operands_and_partials.d_x1[n] += rep_deriv * alpha_dbl / y_dbl;
75 if (!is_constant_struct<T_shape>::value)
76 operands_and_partials.d_x2[n] += rep_deriv * log(y_dbl / sigma_dbl);
77 if (!is_constant_struct<T_scale>::value)
78 operands_and_partials.d_x3[n] -= rep_deriv * alpha_dbl / sigma_dbl;
79 }
80
81 return operands_and_partials.value(cdf_log);
82 }
83 }
84 }
85 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::weibull_cdf_log
return_type< T_y, T_shape, T_scale >::type weibull_cdf_log(const T_y &y, const T_shape &alpha, const T_scale &sigma)
Definition: weibull_cdf_log.hpp:29
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
constants.hpp
check_finite.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.